/*
 * This file contains the C library functions for the Mathomatic library.
 * Refer to this, if you are going to use the Mathomatic code in other projects.
 *
 * Be sure and define "LIBRARY" when compiling the Mathomatic code for this.
 * And be sure to call "clear_all()" after completing each group of operations,
 * otherwise the equation spaces will fill up.
 *
 * Mathomatic Copyright (c) 1996 George Gesslein II.
 */

#include "../includes.h"
#include <signal.h>

/*
 * Initialize Mathomatic.
 * Call this before calling any Mathomatic code.
 *
 * Returns true if successful.
 */
int
matho_init()
{
	init_gvars();
	gfp = stdout;
	if (!init_mem()) {
		return false;
	}
	signal(SIGFPE, fphandler);
	return true;
}

/*
 * Process a Mathomatic command or input equation.
 * Input string is in "input", output string is stored in "*outputp".
 *
 * If this returns true, the command or input was successful,
 * and the resulting equation output is stored in "*outputp".
 * This is a malloc()ed equation text string which must be free()d after use.
 *
 * If this returns false, the command or input failed and a text error
 * message is stored in "*outputp".  The error message must not
 * be free()d.
 *
 * This function works just like typing something into the Mathomatic prompt.
 */
int
matho_process(char *input, char **outputp)
{
	int	i;
	int	rv;

	*outputp = NULL;
	if ((i = setjmp(jmp_save)) != 0) {
		clean_up();	/* Mathomatic processing was interrupted, so do a clean up. */
		if (i == 14) {
			error("Expression too big!");
		}
		*outputp = error_str;
		return false;
	}
	result_str = NULL;
	error_str = NULL;
	set_error_level(input);
	if ((rv = process(input))) {
		*outputp = result_str;
	} else {
		*outputp = error_str;
	}
	return rv;
}

void
fphandler(int sig)
{
	/* a floating point exception occured */
	signal(SIGFPE, fphandler);
	longjmp(jmp_save, 2);
}

/*
 * Exit this program and return to the Operating System.
 */
void
exit_program(int exit_value)
{
	exit(exit_value);
}
